

if __name__ == '__main__':
    from a7p import load
    from a7p.exceptions import A7PProtoValidationError, _extract_protovalidate_violations

    # Main execution block
    with open(r"./broken.a7p", 'rb') as fp:
        try:
            a7p = load(fp, validate_=True)
        except A7PProtoValidationError as e:
            for v in e.violations:
                print(f"Field Path: {v.path}")
                print(f"Constraint ID: {v.value}")
                print(f"Message: {v.reason}")
                print("---")



if __name__ == '__main__':
    from a7p import load
    from a7p.spec_validator import validate_spec
    from a7p.protovalidate import validate as proto_validate
    from a7p.yupy_schema import validate as yupy_validate
    from yupy import ValidationError
    import timeit
    import tqdm
    from pathlib import Path


    # test index switch distance
    with open("example2.a7p", 'rb') as fp:
        p = load(fp, fail_fast=True, validate_=False)
        try:
            yupy_validate(p, fail_fast=False)
        except ValidationError as err:
            for msg in err.messages:
                print(msg)

    d = Path('../a7p-lib/gallery').rglob("*")
    fs = [p for p in d if p.is_file()]
    errs = []
    for f in tqdm.tqdm(fs):
        with open(f, 'rb') as fp:
            p = load(fp, fail_fast=True, validate_=False)
            try:
                yupy_validate(p, fail_fast=False)
            except ValidationError as err:
                errs.append(err)

    print("errs:", len(errs))

    for e in errs[0].errors:
        print("Violation:")
        print(f"\tPath\t:\t{e.path}")
        print(f"\tValue\t:\t{e.invalid_value!r}")
        print(f"\tReason\t:\t{e.constraint.format_message}")

    def speedtest(validator):
        for f in tqdm.tqdm(fs, desc=validator.__name__):
            # with open("example.a7p", 'rb') as fp:
            with open(f, 'rb') as fp:
                p = load(fp, fail_fast=True)
                try:
                    validator(p)
                except ValidationError as err:
                    pass

    def v_old():
        speedtest(validate_spec)

    def v_new():
        speedtest(yupy_validate)

    def v_pro():
        speedtest(proto_validate)

    num = 1
    told = timeit.timeit(v_old, number=num)
    print("spec validator", told)  # 0.0601s

    tnew = timeit.timeit(v_new, number=num)
    print("yupy validator", tnew)  # 0.0059s

    tpro = timeit.timeit(v_pro, number=num)
    print("protovalidate", tpro)  # 3.4331s



from a7p import exceptions, load
from a7p.logger import color_print, logger
from a7p.recover.recover_process import attempt_to_recover


def _example():
    try:
        with open("broken.a7p", "rb") as fp:
            try:
                # trying to load file data to payload
                load(fp, fail_fast=False)
            except exceptions.A7PValidationError as err:
                color_print("Violations found:", levelname="ERROR")
                for v in err.all_violations:
                    color_print(v.format(), levelname="WARNING")
                attempt_to_recover(err)
            else:
                logger.info("No violations found")

    except IOError as e:
        print("Error: %s" % e)

if __name__ == '__main__':
    _example()







from yupy import mixed, string, number, array, mapping
from yupy.validation_error import ValidationError

if __name__ == "__main__":

    s = string().max(5).min(2).lowercase()
    n = number().required().integer().ge(100).le(10).multiple_of(30)
    l = array().required().of(s).min(2)

    s.validate("ab")
    n.validate(60)

    shp = mapping().shape(
        {
            "email": string().email().required(),
            "s": s,
            "n": n,
            "shp": mapping()
            .shape(
                {
                    "s": s,
                    "n": n,
                    # 'l': l,
                    "o": mapping().shape({"n": l}),
                }
            )
            .required(),
        }
    )

    # shp.validate({'s': "ab", 'n': 60})
    try:
        shp.validate(
            {
                "email": "a@gmail.com",
                "s": "wd",
                "n": 60,
                "shp": {"n": "a", "l": ["ab", "b"], "o": {"n": ["g", "g"]}},
            }, False
        )
    except ValidationError as err:

        for e in err.errors:
            print(str(e.path or "<None>") + ':')
            print('  ', e.constraint.format_message)

    m = mixed().one_of(['G1', 'G7'])
    m.validate('F')



import a7p

with open("bc_ok.a7p", "rb") as fp:
    payload = a7p.load(fp, validate_=True)

del payload.profile.distances[:]
payload.profile.distances[:] = [10000000, ]
del payload.profile.switches[3:]
payload.profile.b_weight = 2000000
payload.profile.c_muzzle_velocity = 100000
payload.profile.short_name_top = "abcdefghij"
# payload.profile.c_zero_distance_idx = 100000
# payload.profile.coef_rows.append([])

with open("broken.a7p", "wb") as fp:
    a7p.dump(payload, fp, False)
